@@ -31,7 +31,8 @@ def encode(value,
            max_iter=None,
            use_decimal=False,
            numeric_keys=False,
-           use_base85=False):
+           use_base85=False,
+           fail_safe=None):
     """Return a JSON formatted representation of value, a Python object.
 
     :param unpicklable: If set to False then the output will not contain the
@@ -66,6 +67,13 @@ def encode(value,
 
         NOTE: A side-effect of the above settings is that float values will be
         converted to Decimal when converting to json.
+    :param use_base85:
+        If possible, use base85 to encode binary data. Base85 bloats binary data
+        by 1/4 as opposed to base64, which expands it by 1/3. This argument is
+        ignored on Python 2 because it doesn't support it.
+    :param fail_safe: If set to a function exceptions are ignored when pickling
+        and if a exception happens the function is called and the return value
+        is used as the value for the object that caused the error
 
     >>> encode('my string') == '"my string"'
     True
@@ -76,10 +84,6 @@ def encode(value,
     >>> encode({'foo': [1, 2, [3, 4]]}, max_depth=1)
     '{"foo": "[1, 2, [3, 4]]"}'
 
-    :param use_base85:
-        If possible, use base85 to encode binary data. Base85 bloats binary data
-        by 1/4 as opposed to base64, which expands it by 1/3. This argument is
-        ignored on Python 2 because it doesn't support it.
     """
     backend = backend or json
     context = context or Pickler(
@@ -92,7 +96,8 @@ def encode(value,
             max_iter=max_iter,
             numeric_keys=numeric_keys,
             use_decimal=use_decimal,
-            use_base85=use_base85)
+            use_base85=use_base85,
+            fail_safe=fail_safe)
     return backend.encode(context.flatten(value, reset=reset))
 
 
@@ -108,7 +113,8 @@ class Pickler(object):
                  max_iter=None,
                  numeric_keys=False,
                  use_decimal=False,
-                 use_base85=False):
+                 use_base85=False,
+                 fail_safe=None):
         self.unpicklable = unpicklable
         self.make_refs = make_refs
         self.backend = backend or json
@@ -136,6 +142,9 @@ class Pickler(object):
             self._bytes_tag = tags.B64
             self._bytes_encoder = util.b64encode
 
+        # ignore exceptions
+        self.fail_safe = fail_safe
+
     def reset(self):
         self._objs = {}
         self._depth = -1
@@ -214,48 +223,63 @@ class Pickler(object):
         return self._flatten(obj)
 
     def _flatten(self, obj):
+
+        #########################################
+        # if obj is nonrecursive return immediately
+        # for performance reasons we don't want to do recursive checks
+        if PY2 and isinstance(obj, types.FileType):
+            return self._flatten_file(obj)
+
+        if util.is_bytes(obj):
+            return self._flatten_bytestring(obj)
+
+        if util.is_primitive(obj):
+            return obj
+
+        # Decimal is a primitive when use_decimal is True
+        if self._use_decimal and isinstance(obj, decimal.Decimal):
+            return obj
+        #########################################
+
         self._push()
         return self._pop(self._flatten_obj(obj))
 
+    def _max_reached(self):
+        return self._depth == self._max_depth
+
     def _flatten_obj(self, obj):
         self._seen.append(obj)
-        max_reached = self._depth == self._max_depth
 
-        in_cycle = (
-            max_reached or (
-                not self.make_refs
-                and id(obj) in self._objs
-            )) and not util.is_primitive(obj)
-        if in_cycle:
-            # break the cycle
-            flatten_func = repr
-        else:
-            flatten_func = self._get_flattener(obj)
+        max_reached = self._max_reached()
 
-        if flatten_func is None:
-            self._pickle_warning(obj)
-            return None
+        try:
 
-        return flatten_func(obj)
+            in_cycle = _in_cycle(obj, self._objs, max_reached, self.make_refs)
+            if in_cycle:
+                # break the cycle
+                flatten_func = repr
+            else:
+                flatten_func = self._get_flattener(obj)
+
+            if flatten_func is None:
+                self._pickle_warning(obj)
+                return None
+
+            return flatten_func(obj)
+
+        except (KeyboardInterrupt, SystemExit) as e:
+            raise e
+        except Exception as e:
+            if self.fail_safe is None:
+                raise e
+            else:
+                return self.fail_safe(e)
 
     def _list_recurse(self, obj):
         return [self._flatten(v) for v in obj]
 
     def _get_flattener(self, obj):
 
-        if PY2 and isinstance(obj, types.FileType):
-            return self._flatten_file
-
-        if util.is_bytes(obj):
-            return self._flatten_bytestring
-
-        if util.is_primitive(obj):
-            return lambda obj: obj
-
-        # Decimal is a primitive when use_decimal is True
-        if self._use_decimal and isinstance(obj, decimal.Decimal):
-            return lambda obj: obj
-
         list_recurse = self._list_recurse
 
         if util.is_list(obj):
@@ -295,14 +319,24 @@ class Pickler(object):
     def _ref_obj_instance(self, obj):
         """Reference an existing object or flatten if new
         """
-        if self._mkref(obj):
-            # We've never seen this object so return its
-            # json representation.
+        if self.unpicklable:
+            if self._mkref(obj):
+                # We've never seen this object so return its
+                # json representation.
+                return self._flatten_obj_instance(obj)
+            # We've seen this object before so place an object
+            # reference tag in the data. This avoids infinite recursion
+            # when processing cyclical objects.
+            return self._getref(obj)
+        else:
+            max_reached = self._max_reached()
+            in_cycle = _in_cycle(obj, self._objs, max_reached, False)
+            if in_cycle:
+                # A circular becomes None.
+                return None
+
+            self._mkref(obj)
             return self._flatten_obj_instance(obj)
-        # We've seen this object before so place an object
-        # reference tag in the data. This avoids infinite recursion
-        # when processing cyclical objects.
-        return self._getref(obj)
 
     def _flatten_file(self, obj):
         """
@@ -518,7 +552,8 @@ class Pickler(object):
                     # We've never seen this object before so pickle it in-place.
                     # Create an instance from the factory and assume that the
                     # resulting instance is a suitable examplar.
-                    value = self._flatten(handlers.CloneFactory(factory()))
+                    value = self._flatten_obj_instance(
+                        handlers.CloneFactory(factory()))
                 else:
                     # We've seen this object before.
                     # Break the cycle by emitting a reference.
@@ -612,6 +647,14 @@ class Pickler(object):
             warnings.warn(msg)
 
 
+def _in_cycle(obj, objs, max_reached, make_refs):
+    return (
+        max_reached or (
+            not make_refs and id(obj) in objs
+        )
+    ) and not util.is_primitive(obj)
+
+
 def _mktyperef(obj):
     """Return a typeref dictionary
 
